// Rouslan Synth 2011-2015
#include "declarations.h"
#include "lcd.h"

#if SIMULATOR
static char LCD_DATAPORT;
static char LCD_CMDPORT;
static char UI_BUTTONPORT;
//static char PORTC;
//static char ADCH;
static char ADCSRA;
#else
#define LCD_DATAPORT	PORTD
#define LCD_CMDPORT		PORTC
#define UI_BUTTONPORT	PINB
#endif

inline void send_char1(char c) {
#if SIMULATOR
	putchar(c);
#endif
	LCD_CMDPORT  = LCD_DISABLE | LCD_CHAR;
	LCD_DATAPORT = c << 1;
}

inline void send_char2() {
	LCD_CMDPORT  = LCD_ENABLE | LCD_CHAR;
}

inline void send_cmd1(u8 c) {
#if SIMULATOR
	if(c & LCD_CLEAR)
		putchar('\n');
#endif
	LCD_CMDPORT  = LCD_DISABLE | LCD_COMMAND;
	LCD_DATAPORT = c;
}

inline void send_cmd2() {
	LCD_CMDPORT = LCD_ENABLE | LCD_COMMAND;
}

inline void send_cmd_cursor_pos(u8 loc) {
#if SIMULATOR
	u8 i;
	loc -= LCD_LINE2;
	putchar('\n');
	for(i=0; i<loc; i++)
		putchar(' ');
#endif
	LCD_CMDPORT  = LCD_DISABLE | LCD_COMMAND | LCD_LOCSEL;
	LCD_DATAPORT = loc << 1;
}

inline void send_cmd_cursor_pos2() {
	LCD_CMDPORT  = LCD_ENABLE | LCD_COMMAND | LCD_LOCSEL;
}

char *string;
char values[4][4];
u8 val_idx;
u8 carret_pos;
u8 val_char_pos = 0;
u8 val_print_all;
u8 cycle;
u8 mode;

enum {
	LCD_IDLE,
	LCD_STRING,
	LCD_VALUE,
	LCD_INIT,
};

void lcd_set_carret(u8 pos)
{
	carret_pos = pos;
}
u8 lcd_busy()
{
	return mode != LCD_IDLE;
}

u8 _signed_vals;

void _update_value(u8 pos, u8 val)
{
	u8 i = 0, neg = 0;
	values[pos][0] = ' ';
	values[pos][1] = ' ';
	values[pos][2] = ' ';

	if(_signed_vals && (s8)val < 0) {
		val = -(s8)val;
		neg = 1;
	}

	do {
		values[pos][3-i] =  val % 10 + '0';
		val /= 10;
		i++;
	} while (val > 0);

	if(neg)
		values[pos][3-i] = '-';
}

u8 last_v3, last_idx_update = 4, last_carret_pos = 4, lcd_reprint_all_values = 1;

void lcd_print_values(u8 v0, u8 v1, u8 v2, u8 v3, u8 signed_vals)
{
	_signed_vals = signed_vals;

	if(carret_pos != last_carret_pos || lcd_reprint_all_values == 1) {
		_update_value(0, v0);
		_update_value(1, v1);
		_update_value(2, v2);
		_update_value(3, v3);

		val_print_all = 1;
		val_idx = 0;
		lcd_reprint_all_values = 0;
	} else {
		if(carret_pos == 0) _update_value(0, v0);
		else if(carret_pos == 1) _update_value(1, v1);
		else if(carret_pos == 2) _update_value(2, v2);
		else if(carret_pos == 3) _update_value(3, v3);

		val_idx = carret_pos;
		val_print_all = 0;
	}

	if(mode != LCD_STRING) {
		cycle = 0;
		mode = LCD_VALUE;
	}
}

void lcd_print_menu(char* str)
{
	cycle = 0;
	string = str;
	mode = LCD_STRING;
}

inline void _print_value() {
	if(cycle > 11) {
		val_char_pos = 0;
		if(val_print_all && val_idx < 3) {
			val_idx++;
			cycle = 2;
		} else {
			last_carret_pos = carret_pos;
			mode = LCD_IDLE;
			return;
		}
	}
	switch(cycle) {
		case  0: send_cmd_cursor_pos(LCD_LINE2 + 5*val_idx); break;
		case  1: send_cmd_cursor_pos2(); break;
		case  2:
		case  4:
		case  6:
		case  8: send_char1(values[val_idx][val_char_pos++]); break;
		case  3:
		case  5:
		case  7:
		case  9:
		case 11: send_char2(); break;
		case 10: send_char1(carret_pos == val_idx ? '<' : ' '); break;
	}
	cycle++;
}

inline void _print_string() {
	if(cycle > 49) {
		send_char2();
		mode = LCD_IDLE;
		cycle = 0;
		lcd_reprint_all_values = 1;
		return;
	}
	switch(cycle) {
		case 0:	send_cmd1(LCD_CLEAR);//send_cmd_cursor_pos(0); break;
		case 1:	send_cmd2(); break;
		default:
			if(cycle & 1)
				send_char2();
			else {
				u8 c = string[(cycle-2)>>1];
				if(c != 0)
					send_char1(c);
				else
					cycle = 49;
			}
			break;
	}
	cycle++;
}

inline void _init()
{
	if(cycle > 5) {
		mode = LCD_IDLE;
		return;
	}
	switch(cycle) {
		case 0:	send_cmd1(LCD_2LINE); break;
		case 1:	send_cmd2(); break;
		case 2:	send_cmd1(LCD_ON); break;
		case 3:	send_cmd2(); break;
		case 4:	send_cmd1(LCD_INCREMENT); break;
		case 5:	send_cmd2(); break;
	}
	cycle++;
}

void lcd_update()
{
	switch(mode) {
		case LCD_IDLE:
			return;
		case LCD_VALUE: _print_value(); break;
		case LCD_STRING: _print_string(); break;	
		case LCD_INIT: _init(); break;
	}
}

void lcd_init()
{
	cycle = 0;
	mode = LCD_INIT;
}
