// Rouslan Synth 2011-2015
#define _CRT_SECURE_NO_DEPRECATE
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <mmsystem.h>
#include <stdlib.h>
#include <stdio.h>
#include "midi.h"
#include "synth.h"

#pragma warning (disable:4996)
#pragma comment(lib, "winmm.lib")

void control();
void loadRom();
void genNotes();

#define BUF_LEN 2048
#define NBUF 2
#define FILE_OUT if(1)

u8 synth_shutdown;
HANDLE bufferReady, bufferEmpty;

void CALLBACK MidiInProc(HMIDIIN hMidiIn, UINT wMsg, DWORD_PTR dwInstance, DWORD_PTR dwParam1, DWORD_PTR dwParam2)
{
	u8 status, param[2];

	status = (u8)(dwParam1 & 0xFF);

	if(wMsg != MIM_DATA || status >= 0xF0)
		return;

	param[0] = (u8)((dwParam1>>8) & 0xFF);
	param[1] = (u8)((dwParam1>>16) & 0xFF);
	
	//printf("%X %d %d\n", status, param[0], param[1]);
	midi_write(status, param[0], param[1]);
}

void CALLBACK waveOutProc(HWAVEOUT hwo, UINT uMsg, DWORD_PTR waveDone, DWORD_PTR dwParam1, DWORD_PTR dwParam2)
{
	if(uMsg == WOM_DONE)
		SetEvent((HANDLE)waveDone);
}

int main(int argc, char *argv[])
{
	int			i, c = 0;
	u8			data[NBUF][BUF_LEN];
	// Wave
	WAVEHDR		waveHdr[NBUF];
	HWAVEOUT	hWaveOut;
	WAVEFORMATEX wfx;
	HANDLE		waveDone;
	// MIDI
	MIDIINCAPS	pmic;
	HMIDIIN		hMidiIn;
	u8			midiConnected = 0;

	FILE		*fp;

	// MIDI in
	if(!midiInGetNumDevs() || midiInOpen(&hMidiIn, 0, (DWORD_PTR)(&MidiInProc), 0, CALLBACK_FUNCTION)) {
		puts("Couldn't connect to a MIDI in device.");
	} else {
		midiInGetDevCaps(0, &pmic, sizeof(MIDIINCAPS));
		wprintf(L"Connected to %s.\n", pmic.szPname);
		midiInStart(hMidiIn);
		midiConnected = 1;
	}

	// Wave out
	wfx.nSamplesPerSec = SAMPLE_RATE;
	wfx.wBitsPerSample = 8;
	wfx.nChannels = 1;
	wfx.cbSize = 0;
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nBlockAlign = (wfx.wBitsPerSample * wfx.nChannels) / 8;
	wfx.nAvgBytesPerSec = wfx.nBlockAlign * wfx.nSamplesPerSec;

	waveDone = CreateEvent(0, 0, 0, 0);

	if(waveOutOpen(&hWaveOut, WAVE_MAPPER, &wfx, (DWORD_PTR)waveOutProc, (DWORD)waveDone, CALLBACK_FUNCTION) != MMSYSERR_NOERROR) {
		puts("Cannot open wave mapper device");
		return 1;
	}

	for(i=0; i<NBUF; i++) {
		waveHdr[i].dwBufferLength = BUF_LEN;
		waveHdr[i].dwBytesRecorded = 0;
		waveHdr[i].dwFlags = 0;
		waveHdr[i].dwLoops = 0;
		waveHdr[i].dwUser = 0;
		waveHdr[i].lpData = (LPSTR)data[i];
		waveHdr[i].lpNext = 0;

		waveOutPrepareHeader(hWaveOut, &waveHdr[i], sizeof(WAVEHDR));
	}

	// Boot up the synth
	loadRom();
	synth_main();

	FILE_OUT fp = fopen("out.raw", "wb");
	
	// Main loop
	while(!GetAsyncKeyState(VK_ESCAPE)) {
		if(!midiConnected)
			genNotes();

		for(i=0; i<NBUF; i++) {
			int s;
			for(s=0; s<BUF_LEN; s++) {
				data[i][s] = TIMER0_COMPA_vect();
			}
			FILE_OUT {
				if(ftell(fp) < 64*1024)
					fwrite(data[i], sizeof(u8), BUF_LEN, fp);
			}
			if(waveOutWrite(hWaveOut, waveHdr + i, sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
				break;
			// Buffer up NBUF - 1 packets
			if(c == NBUF - 1)
				WaitForSingleObject(waveDone, INFINITE);
			else
				c++;
		}
	}

	FILE_OUT fclose(fp);

	// Shutdown
	for(i=0; i<NBUF; i++) 
		if(waveHdr[i].dwFlags & WHDR_PREPARED)
			waveOutUnprepareHeader(hWaveOut, &waveHdr[i], sizeof(WAVEHDR));

	CloseHandle(waveDone);
	waveOutClose(hWaveOut);

	midiInClose(hMidiIn);

	return 0;
}

u8 eeprom[128];

void loadRom()
{
	int l = 0;
	char line[64], hex[3];
	FILE *fp = fopen("patch_save.hex", "r");
	if(!fp) {
		puts("Couldn't find eeprom dump.\n");
		return;
	}

	hex[2] = 0;

	while(fgets(line, 64, fp)) {
		int i;
		for(i=0; i<16; i++) {
			hex[0] = line[ 9 + 2*i];
			hex[1] = line[10 + 2*i];
			eeprom[i + 16*l] = (u8)strtol(hex, 0, 16);
		}
		l++;
	}
	fclose(fp);
}

int noteDelay = 0;

void genNotes()
{
	if(noteDelay++ == 0)
		midi_write(0x90, 33, 100);
	else if(noteDelay == 12)
		midi_write(0x80, 33, 0); // Play A1
	else if(noteDelay == 15)
		noteDelay = 0;
}