#pragma once
#include "declarations.h"
#include "midi.h"

extern const u16 note2offset[];
extern const s8 sine[];

typedef struct {
	u8			value;
	s8			rate;
} ctrl_point;

#define ENV_POINTS 4

typedef struct {
	ctrl_point	pts[ENV_POINTS]; 
	u8			point;
	u8			value;
} envelope;

#define synth_env_reset(e) { e.point = 0; e.value = e.pts[0].value; }

typedef enum {
	SQUARE,
	SAWTOOTH,
	SINE,
	NOISE
} osc_types;

typedef enum {
	PLAYING,
	DECAYING,
	OFF
} note_state;

typedef struct {
	u16			ds;			// sample delta
	u16			target_ds;	// when sliding between tones
	//u8			unison_shift;
	u16			cur_sample;
	osc_types	type;
	u8			tweak; // duty for square wave, dc bias for sine
	s8			note_offset;
	envelope	env_vol;
} oscillator;

typedef struct {
	oscillator	osc[2];
	s16			bend;
	s16			bend_modulation;
	u8			notes[MIDI_CMD_BUFFER];	// last notes played
	u8			note_idx;
	u8			vol;
	u8			mod;
	u8			ring;
	note_state	state;
	u8			slide_rate;
	// low-pass cotrol
	u8			breadth;
	envelope	env_low_pass;
	u8			inst_low_pass;
	s16			output;
	s16			last_output;
} channel;

typedef enum {
	CTRL_OSC_SHAPE,
	CTRL_OSC0_ENV_VOL_VALUE,
	CTRL_OSC0_ENV_VOL_RATE,
	CTRL_OSC1_ENV_VOL_VALUE,
	CTRL_OSC1_ENV_VOL_RATE,
	CTRL_LP_ENV_VOL_VALUE,
	CTRL_LP_ENV_VOL_RATE,
	CTRL_TRANSPOSE_AND_RING_MOD,
	CTRL_PATCH_SAVE,
	CTRL_PATCH_RESTORE,
	CTRL_N
} ctrl_banks;

#define N_CHAN 1
extern channel chan[N_CHAN];
extern u8 ctrl_bank;
extern u8 cur_ctrl;
extern u8 cur_patch;

void synth_init();
void synth_main();
void synth_update();
void synth_sample();
void synth_update_controls(u8 val);
void compute_ds(channel* c, u8 set_cur_ds);
void synth_save();
void synth_load();