// Rouslan Synth 2011-2015
#include "declarations.h"
#include "ui.h"
#include "lcd.h"
#include "midi.h"

#if SIMULATOR
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
static u8 ADCH = 0;
#endif

enum menu_items
{
	MENU_MIDI_DEBUG = CTRL_N,
	MENU_N
};

u8 menu_item;
u8 menu_refresh_val;
u8 menu_changed;
u8 left_state;
u8 right_state;
u8 left_held;
u8 right_held;
u8 last_adc;

void menu_set(u8 menu)
{
	if(menu > MENU_N)
		menu = MENU_N - 1;
	else if (menu == MENU_N)
		menu = 0;

	if(menu_item == menu)
		menu_refresh_val = 1;
	else {
		menu_item = menu;
		menu_changed = 1;
	}
}

u8 last_adc;

void menu_init()
{
	last_adc = ADCH;
	menu_changed = 1;
}

void menu_update_val(u8 val)
{
	lcd_set_carret(cur_ctrl);
	menu_refresh_val = 1;
}

void menu_update()
{
	oscillator *o;
#if SIMULATOR
	static u8 adc;
	left_state  = left_state  << 1 | (GetKeyState(VK_LEFT) & 0x80 ? 1:0);
	right_state = right_state << 1 | (GetKeyState(VK_RIGHT) & 0x80 ? 1:0);
	if(GetKeyState(VK_UP) & 0x80) adc++;
	if(GetKeyState(VK_DOWN) & 0x80) adc--;
#else
	u8 adc = ADCH;
	left_state  = left_state  << 1 | (UI_BUTTONPORT & BUTTON_LEFT  ? 0:1);
	right_state = right_state << 1 | (UI_BUTTONPORT & BUTTON_RIGHT ? 0:1);
#endif

	if(left_state && right_state) { // Save or restore the patch when both buttons are pressed
		if(menu_item == CTRL_PATCH_SAVE) {
			LED_RED_ON(3);
			synth_save();
			menu_set(0);
		} else if(menu_item == CTRL_PATCH_RESTORE) {
			LED_RED_ON(3);
			synth_load();
			menu_set(0);
		}
	} else
	// Left Button
	if(left_state) {
		if((left_state & 0xF) == 0xC) { // Release
			if(left_held < HOLD_DURATION) {
				cur_ctrl = cur_ctrl - 1 & 3;
				lcd_set_carret(cur_ctrl);
				menu_refresh_val = 1;
				if(cur_ctrl == 3) {
					menu_set(menu_item - 1);
				}
			}
			left_held = 0;
		} else if(left_held == HOLD_DURATION) { // Hold
			menu_set(menu_item - 1);
		}
		left_held++;
	} else
		left_held = 0;

	// Right Button
	if(right_state) {
		if((right_state & 0xF) == 0xC) { // Release
			if(right_held < HOLD_DURATION) {
				cur_ctrl = cur_ctrl + 1 & 3;
				lcd_set_carret(cur_ctrl);
				menu_refresh_val = 1;
				if(cur_ctrl == 0) {
					menu_set(menu_item + 1);
				}
			}
			right_held = 0;
		} else if(right_held == HOLD_DURATION) { // Hold
			menu_set(menu_item + 1);
		}
		right_held++;
	} else
		right_held = 0;

	if(adc != last_adc) {
		synth_update_controls(adc);
		last_adc = adc;
	}


	if(menu_changed) {
		LED_GREEN_ON(1);

		if(menu_item < CTRL_N)
			ctrl_bank = menu_item;

		midi_debug = 0;

		switch(menu_item) {
			case CTRL_OSC_SHAPE:				lcd_print_menu("Osc 0 S+D     Osc 1 S+D");	break;
			case CTRL_OSC0_ENV_VOL_VALUE:		lcd_print_menu("Osc 0 Envelope Vals");		break;
			case CTRL_OSC0_ENV_VOL_RATE:		lcd_print_menu("Osc 0 Envelope Rates");		break;
			case CTRL_OSC1_ENV_VOL_VALUE:		lcd_print_menu("Osc 1 Envelope Vals");		break;
			case CTRL_OSC1_ENV_VOL_RATE:		lcd_print_menu("Osc 1 Envelope Rates");		break;
			case CTRL_LP_ENV_VOL_VALUE:			lcd_print_menu("Lowpass Envelope Vals");	break;
			case CTRL_LP_ENV_VOL_RATE:			lcd_print_menu("Lowpass Envelope Rates");	break;
			case CTRL_TRANSPOSE_AND_RING_MOD:	lcd_print_menu(" Osc Trnsp  Slide Ring");	break;
			case CTRL_PATCH_SAVE:				lcd_print_menu("Patch Save");				break;
			case CTRL_PATCH_RESTORE:			lcd_print_menu("Patch Restore");			break;
			case MENU_MIDI_DEBUG:				lcd_print_menu("MIDI Message Log "); midi_debug = 1; break;
		}
		menu_changed = 0;
		menu_refresh_val = 1;
		return;
	}

	if(lcd_busy())
		return;

	o = chan[0].osc;

	if(menu_refresh_val) {
		menu_refresh_val = 0;
		switch(menu_item) {
			case CTRL_OSC_SHAPE:				lcd_print_values(o[0].type, o[0].tweak, o[1].type, o[1].tweak, 0);	 break;
			case CTRL_OSC0_ENV_VOL_VALUE:		lcd_print_values(o[0].env_vol.pts[0].value, o[0].env_vol.pts[1].value, o[0].env_vol.pts[2].value, o[0].env_vol.pts[3].value, 0); break;
			case CTRL_OSC0_ENV_VOL_RATE:		lcd_print_values(o[0].env_vol.pts[0].rate,  o[0].env_vol.pts[1].rate,  o[0].env_vol.pts[2].rate,  o[0].env_vol.pts[3].rate, 1); break;
			case CTRL_OSC1_ENV_VOL_VALUE:		lcd_print_values(o[1].env_vol.pts[0].value, o[1].env_vol.pts[1].value, o[1].env_vol.pts[2].value, o[1].env_vol.pts[3].value, 0); break;
			case CTRL_OSC1_ENV_VOL_RATE:		lcd_print_values(o[1].env_vol.pts[0].rate,  o[1].env_vol.pts[1].rate,  o[1].env_vol.pts[2].rate,  o[1].env_vol.pts[3].rate, 1); break;
			case CTRL_LP_ENV_VOL_VALUE:			lcd_print_values(chan[0].env_low_pass.pts[0].value, chan[0].env_low_pass.pts[1].value, chan[0].env_low_pass.pts[2].value, chan[0].env_low_pass.pts[3].value, 0); break;
			case CTRL_LP_ENV_VOL_RATE:			lcd_print_values(chan[0].env_low_pass.pts[0].rate, chan[0].env_low_pass.pts[1].rate, chan[0].env_low_pass.pts[2].rate, chan[0].env_low_pass.pts[3].rate, 1); break;
			case CTRL_TRANSPOSE_AND_RING_MOD:	lcd_print_values(o[0].note_offset, o[1].note_offset, chan[0].slide_rate, chan[0].ring, 1); break;
			case CTRL_PATCH_SAVE:				
			case CTRL_PATCH_RESTORE:			lcd_print_values(cur_patch, 0,0,0,0); break;
		}
	}
}
